﻿#ifndef _CLIXUP_OTDSDK_H_
#define _CLIXUP_OTDSDK_H_

//User should implement following #define / typedef for local platform
//  X_Void as void
//  X_UInt8 as 8 bit unsigned integer
//  X_Int8 as 8 bit signed integer
//  X_UInt16 as 16 bit unsigned integer
//  X_Int16 as 16 bit signed integer
//  X_UInt32 as 32 bit unsigned integer
//  X_Int32 as 32 bit signed integer
//  X_UInt64 as 64 bit unsigned integer
//  X_Float32 as 32 bit floating point
//  X_Float64 as 64 bit floating point
//  X_Char as ASCII char
//  X_WChar as Unicode char
//  X_Null as a null pointer

#ifdef __GNUC__
#if defined(__i386)
#define OTD_API __attribute((__stdcall__))
#endif
#endif

#ifdef _MSC_VER
#define OTD_API __stdcall
#endif

#ifndef OTD_API
#define OTD_API
#endif

#ifdef __cplusplus
#define OTD_CSTYLE_API(type) extern "C" type OTD_API
#else
#define OTD_CSTYLE_API(type) type OTD_API
#endif

typedef X_Void *X_Handle;

//Structures package alligns as 1 Byte for transfer and storage.
#pragma pack(1)

#define DIAGNOSIS_DATA_SIZE         0x3F
typedef struct _DiagnosisData
{
    X_UInt8 data[DIAGNOSIS_DATA_SIZE];
}
DiagnosisData;

typedef X_UInt8 RawtouchDataStatus;
#define RawtouchDataStatus_Up   ((RawtouchDataStatus)0x00)
#define RawtouchDataStatus_Down ((RawtouchDataStatus)0x01)

typedef struct _RawtouchData
{
    RawtouchDataStatus status;
    X_UInt8 id;
    X_Float32 x;
    X_Float32 y;
    X_Float32 width;
    X_Float32 height;
    X_UInt16 scanTime;
}
RawtouchData;

typedef X_UInt8 TouchDataStatus;
#define TouchDataStatus_Up      ((TouchDataStatus)0x00)
#define TouchDataStatus_Down    ((TouchDataStatus)0x01)

typedef struct _TouchData
{
    TouchDataStatus status;
    X_UInt8 id;
    X_Float32 x;
    X_Float32 y;
    X_Float32 width;
    X_Float32 height;
    X_UInt16 scanTime;
}
TouchData;

typedef X_UInt8 VirtualKeyDataStatus;
#define VirtualKeyDataStatus_Down   ((VirtualKeyDataStatus)0x01)
#define VirtualKeyDataStatus_Up     ((VirtualKeyDataStatus)0x00)

typedef struct _VirtualKeyData
{
    X_UInt8 keyValue;
    VirtualKeyDataStatus keyStatus;
}
VirtualKeyData;

typedef struct _CalibrationSource
{
    struct
    {
        X_Float32 x;
        X_Float32 y;
    } touchPoint;
    struct
    {
        X_Float32 x;
        X_Float32 y;
    } displayPoint;
}
CalibrationSource;

typedef struct _DisplayRectangle
{
    X_Float32 minX;
    X_Float32 maxX;
    X_Float32 minY;
    X_Float32 maxY;
}
DisplayRectangle;

#define KEY_SIZE                25
typedef struct _ProductKey
{
    X_UInt8 data[KEY_SIZE];
}
ProductKey;

typedef X_UInt8 TouchScreenHidMode;
#define TouchScreenHidMode_Mouse            ((TouchScreenHidMode)0x00)     // Report HID data as Mouse.
#define TouchScreenHidMode_SingleTouch      ((TouchScreenHidMode)0x01)     // Report HID data as Single Touch.
#define TouchScreenHidMode_MultiTouch       ((TouchScreenHidMode)0x02)     // Report HID data as Multi Touch.

typedef X_UInt8 TouchScreenDataMode;
#define TouchScreenDataMode_Normal          ((TouchScreenDataMode)0x00)     // Report HID data and VirtualKey data.
#define TouchScreenDataMode_Touch           ((TouchScreenDataMode)0x01)     // Report Touch data. With calibratoin, unit mm, located by product parameters.
#define TouchScreenDataMode_Rawtouch        ((TouchScreenDataMode)0x02)     // Report Rawtouch data. Without calibration, unit mm, located by product parameters.
#define TouchScreenDataMode_CodedImage      ((TouchScreenDataMode)0x03)     // Report coded image
#define TouchScreenDataMode_Nop             ((TouchScreenDataMode)0x04)     // Report nothing

typedef X_UInt8 TouchScreenWorkingMode;
#define TouchScreenWorkingMode_Data         ((TouchScreenWorkingMode)0x00)  // Touch screen working normally
#define TouchScreenWorkingMode_Command      ((TouchScreenWorkingMode)0x01)  // Touch screen being stand-by for command

#pragma pack()

typedef X_UInt32    OtdResult;
#define OtdResult_Success          ((OtdResult)0)
#define OtdResult_NotImplemented   ((OtdResult)1)
#define OtdResult_UndefinedError   ((OtdResult)-1)

typedef X_Void *TouchScreenNotificationHandle;

typedef X_UInt8 TouchScreenNotifactionAction;
#define TouchScreenNotifactionAction_Arrival         ((TouchScreenNotifactionAction)0x00)
#define TouchScreenNotifactionAction_RemoveComplete  ((TouchScreenNotifactionAction)0x01)

typedef X_Void(OTD_API *TouchScreenNotificationCallbackA)(X_Void* destination, TouchScreenNotifactionAction action, X_Char const* devicePath);
typedef X_Void(OTD_API *TouchScreenNotificationCallbackW)(X_Void* destination, TouchScreenNotifactionAction action, X_WChar const* devicePath);
#ifdef _UNICODE
typedef TouchScreenNotificationCallbackW TouchScreenNotificationCallback;
#else
typedef TouchScreenNotificationCallbackA TouchScreenNotificationCallback;
#endif

OTD_CSTYLE_API(X_Void) OtdGetSdkVersion(X_UInt32* pMajor, X_UInt32* pMinor, X_UInt32* pBuild, X_UInt32* pPrivate);

OTD_CSTYLE_API(OtdResult) OtdCreateTouchScreenNotificationA(TouchScreenNotificationHandle* pNotificationHandle, TouchScreenNotificationCallbackA notification, X_Void* destination);
OTD_CSTYLE_API(OtdResult) OtdCreateTouchScreenNotificationW(TouchScreenNotificationHandle* pNotificationHandle, TouchScreenNotificationCallbackW notification, X_Void* destination);
#ifdef _UNICODE
#define OtdCreateTouchScreenNotification OtdCreateTouchScreenNotificationW
#else
#define OtdCreateTouchScreenNotification OtdCreateTouchScreenNotificationA
#endif

OTD_CSTYLE_API(OtdResult) OtdDeleteTouchScreenNotification(TouchScreenNotificationHandle notificationHandle);

typedef X_Void(OTD_API *DeviceCallbackA)(X_Void* destination, X_Char const* path, X_WChar const* sn);
typedef X_Void(OTD_API *DeviceCallbackW)(X_Void* destination, X_WChar const* path, X_WChar const* sn);
//Brief:
//  An application-defined method for a device callback. Specify this method when calling the OtdEnumTouchScreens method.
//  Parameters:
//      destination:    Pointer to anything for the callback destination.
//      path:           Device path, pointer to a null-terminated string.
//      sn:             Device SN, pointer to a null-terminated string.
//  Return value:
//      None.
#ifdef _UNICODE
typedef DeviceCallbackW DeviceCallback;
#else
typedef DeviceCallbackA DeviceCallback;
#endif

OTD_CSTYLE_API(X_Void) OtdEnumTouchScreensA(DeviceCallbackA callback, X_Void* destination);
OTD_CSTYLE_API(X_Void) OtdEnumTouchScreensW(DeviceCallbackW callback, X_Void* destination);
//Brief:
//  Enumrate all touch screen devices.
//  Parameters:
//      callback:       Pointer to the application-defined method of type DeviceCallback to call back for each valid touch screen device.
//      destination:    Pointer to anything for the callback destination. This pointer would be used as the parameter destination of the callback.
//  Return value:
//      None.
#ifdef _UNICODE
#define OtdEnumTouchScreens OtdEnumTouchScreensW
#else
#define OtdEnumTouchScreens OtdEnumTouchScreensA
#endif

OTD_CSTYLE_API(OtdResult)  OtdOpenTouchScreenA(X_Char const* path, X_Handle* pDevice);
OTD_CSTYLE_API(OtdResult)  OtdOpenTouchScreenW(X_WChar const* path, X_Handle* pDevice);
//Brief:
//  Get a touch screen handle by opening from device path. Applications must close handle by OtdCloseTouchScreen before the end.
//  Parameters:
//      path:           Device path, pointer to a null-terminated string. It it typically got from the parameter path of the parameter callback of the method OtdEnumTouchScreens.
//      pDevice:        Pointer to a X_Handle to receive the touch screen handle. This pointer would be filled if the method completed without error.
//  Return value:
//      OtdResult_Success   -   Success.
//      Otherwise           -   Refer to definition of the type OtdResult.
#ifdef _UNICODE
#define OtdOpenTouchScreen OtdOpenTouchScreenW
#else
#define OtdOpenTouchScreen OtdOpenTouchScreenA
#endif


/*
* OtdGetTouchScreensCount 获取可以使用的触摸屏数目
* 参数：无
* 返回值：
*   返回值即是触摸屏的数目。
*/
OTD_CSTYLE_API(X_Int32)    OtdGetTouchScreenCount(X_Void);

//Brief:
//  Close a touch screen handle which is open.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdCloseTouchScreen(X_Handle device);

//Brief:
// Restart a touch screen.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdRestartTouchScreen(X_Handle device);

//Brief:
//  Get touch screen hid mode.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      pTsdm:          Pointer to a touch screen hid mode refer to definition of TouchScreenHidMode to receive value.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdGetTouchScreenHidMode(X_Handle device, TouchScreenHidMode* pTshm);
//Brief:
//  Set touch screen hid mode.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      tsdm:           A touch screen hid mode refer to definition of TouchScreenHidMode.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdSetTouchScreenHidMode(X_Handle device, TouchScreenHidMode tshm);

//Brief:
//  Get touch screen data mode.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      pTsdm:          Pointer to a touch screen data mode refer to definition of TouchScreenDataMode to receive value.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdGetTouchScreenDataMode(X_Handle device, TouchScreenDataMode* pTsdm);
//Brief:
//  Set touch screen data mode.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      tsdm:           A touch screen data mode refer to definition of TouchScreenDataMode.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdSetTouchScreenDataMode(X_Handle device, TouchScreenDataMode tsdm);

//Brief:
//  Get touch screen working mode.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      pTswm:          Pointer to a touch screen working mode refer to definition of TouchScreenWorkingMode to receive value.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult) OtdGetTouchScreenWorkingMode(X_Handle device, TouchScreenWorkingMode* pTswm);
//Brief:
//  Set touch screen working mode.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      tswm:           A touch screen working mode refer to definition of TouchScreenWorkingMode.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult) OtdSetTouchScreenWorkingMode(X_Handle device, TouchScreenWorkingMode tswm);

//Brief:
//  Get SN from a touch screen which is open.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      sn:             Pointer to a call-allocted buffer to receive SN as a null-terminated string.
//      snSize:         The size of the sn buffer in bytes.
//      pSnSizeRequired:Pointer to a call-allocted variable of type X_UInt32 that receives the required size of the sn buffer. This parameter is optional and can be X_Null.
//  Return value:
//      OtdResult_Success   -   Success.
//      Otherwise           -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdGetTouchScreenSerialNumber(X_Handle device, X_WChar* sn, X_UInt32 snSize, X_UInt32* pSnSizeRequired);

//Brief:
//  An application-defined method for a diagnosis data callback. Specify this method when calling the OtdRegisterDiagnosisCallback method.
//  Parameters:
//      destination:    Pointer to anything for the callback destination.
//      data:           Pointer to a DiagnosisData structure including data for diagnosis.
//  Return value:
//      None.
typedef X_Void(OTD_API *DiagnosisCallback)(X_Void* destination, DiagnosisData const* data);
//Brief:
//  An application-defined method for a rawtouch data callback. Specify this method when calling the OtdRegisterRawtouchCallback method.
//  Parameters:
//      destination:    Pointer to anything for the callback destination.
//      data:           Pointer to a RawtouchData structure including touch data with out coordinate transformation.
//  Return value:
//      None.
typedef X_Void(OTD_API *RawtouchCallback)(X_Void* destination, RawtouchData const* data);
//Brief:
//  An application-defined method for a touch data callback. Specify this method when calling the OtdRegisterTouchCallback method.
//  Parameters:
//      destination:    Pointer to anything for the callback destination.
//      data:           Pointer to a TouchData structure including touch data.
//  Return value:
//      None.
typedef X_Void(OTD_API *TouchCallback)(X_Void* destination, TouchData const* data);
//Brief:
//  An application-defined method for a touch data callback. Specify this method when calling the OtdRegisterVirtualKeyCallback method.
//  Parameters:
//      destination:    Pointer to anything for the callback destination.
//      data:           Pointer to a VirtualKeyData structure including virtual key data.
//  Return value:
//      None.
typedef X_Void(OTD_API *VirtualKeyCallback)(X_Void* destination, VirtualKeyData const* data);

typedef X_Void  *CallbackManagerHandle;
//Brief:
//  Create a callback manager for receiving data by a touch screen handle which is open. Applications must delete callback manager by OtdDeleteCallbackManager before the end, and the touch screen handle must keep open before deleting the callback manager.
//  Parameters:
//      pCallbackManager:   Pointer to a caller-allocated CallbackManagerHandle varible to receive the callback manager handle. This pointer would be filled if the method completed without error.
//      device:             Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdCreateCallbackManager(CallbackManagerHandle* pCallbackManager, X_Handle device);
//Brief:
//  Delete a callback manager which is created.
//  Parameters:
//      callbackManager:    A CallbackManagerHandle that the callback manager which would be deleted.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdDeleteCallbackManager(CallbackManagerHandle callbackManager);
//Brief:
//  Register a diagnosis callback to a callback manager which is created.
//  Parameters:
//      callbackManager:    CallbackManagerHandle that the callback manager which would be used.
//      callback:           Pointer to the application-defined method of type DiagnosisCallback to call back data.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdRegisterDiagnosisCallback(CallbackManagerHandle callbackManager, DiagnosisCallback callback, X_Void* destination);
//Brief:
//  Unregister the diagnosis callback.
//  Parameters:
//      callbackManager:    A CallbackManagerHandle that the callback manager which would be used.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdUnregisterDiagnosisCallback(CallbackManagerHandle callbackManager);
//Brief:
//  Register a rawtouch callback to a callback manager which is created.
//  Parameters:
//      callbackManager:    CallbackManagerHandle that the callback manager which would be used.
//      callback:           Pointer to the application-defined method of type RawtouchCallback to call back data.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdRegisterRawtouchCallback(CallbackManagerHandle callbackManager, RawtouchCallback callback, X_Void* destination);
//Brief:
//  Unregister the rawtouch callback.
//  Parameters:
//      callbackManager:    A CallbackManagerHandle that the callback manager which would be used.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdUnregisterRawtouchCallback(CallbackManagerHandle callbackManager);
//Brief:
//  Register a touch callback to a callback manager which is created.
//  Parameters:
//      callbackManager:    CallbackManagerHandle that the callback manager which would be used.
//      callback:           Pointer to the application-defined method of type TouchCallback to call back data.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdRegisterTouchCallback(CallbackManagerHandle callbackManager, TouchCallback callback, X_Void* destination);
//Brief:
//  Unregister the touch callback.
//  Parameters:
//      callbackManager:    A CallbackManagerHandle that the callback manager which would be used.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdUnregisterTouchCallback(CallbackManagerHandle callbackManager);
//Brief:
//  Register a virtual key callback to a callback manager which is created.
//  Parameters:
//      callbackManager:    CallbackManagerHandle that the callback manager which would be used.
//      callback:           Pointer to the application-defined method of type VirtualKeyCallback to call back data.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdRegisterVirtualKeyCallback(CallbackManagerHandle callbackManager, VirtualKeyCallback callback, X_Void* destination);
//Brief:
//  Unregister the virtual key callback.
//  Parameters:
//      callbackManager:    A CallbackManagerHandle that the callback manager which would be used.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdUnregisterVirtualKeyCallback(CallbackManagerHandle callbackManager);
//Brief:
//  Unregister all callback including diagnosis, rawtouch, touch and virtual key callback.
//  Parameters:
//      callbackManager:    A CallbackManagerHandle that the callback manager which would be used.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdUnregisterAllCallback(CallbackManagerHandle callbackManager);

//Brief:
//  Get product key from touch screen.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      pKey:           Pointer to a caller-allocated ProductKey variable to receive the product key.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdGetProductKey(X_Handle device, ProductKey* pKey);
//Brief:
//  Write custom data to touch screen.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      key:            ProductKey variable to be set into touch screen.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdSetProductKey(X_Handle device, ProductKey key);

//Brief:
//  Reset device calibration parameter.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdCalibrationReset(X_Handle device);
//Brief:
//  Set device calibration parameter by given data for preview. Application should make preview for user to confirm.
//  Parameters:
//      device:                 Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//      displayRectangle:       The display area of one monitor described by any unit (e.g. pixel, inch, millimeter, etc.).
//      calibrationSource:      An array of calibration source, which contented touch point (source position) and display point (destination point). Touch point would be collected by method OtdRegisterRawtouchCallback. Display point is described by the same unit as displayRectangle.
//      calibrationSourceCount: Element count of the array calibrationSource.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdCalibrationFullScreenPreview(X_Handle device, DisplayRectangle displayRectangle, CalibrationSource const calibrationSource[], X_UInt32 calibrationSourceCount);
//Brief:
//  Terminate preview with cancel.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdCalibrationCancelPreview(X_Handle device);
//Brief:
//  Terminate preview with apply.
//  Parameters:
//      device:         Handle to a touch screen. This handle is typically got by OtdOpenTouchScreen.
//  Return value:
//      OtdResult_Success   -   Success.
//      Others              -   Refer to definition of the type OtdResult.
OTD_CSTYLE_API(OtdResult)  OtdCalibrationApplyPreview(X_Handle device);

#endif
